@@ -9,7 +9,7 @@ will load profiles from 3 places (the later will override the former):
 
 #. STIX-shifter interface config file (only when a Kestrel session starts):
 
-    Put your profiles in the STIX-shifter interface config file (YAML):
+    Create the STIX-shifter interface config file (YAML):
 
     - Default path: ``~/.config/kestrel/stixshifter.yaml``.
     - A customized path specified in the environment variable ``KESTREL_STIXSHIFTER_CONFIG``.
@@ -26,6 +26,9 @@ will load profiles from 3 places (the later will override the former):
                     port: 9200
                     selfSignedCert: false # this means do NOT check cert
                     indices: host101
+                    options:  # options section only needed when using a dialect
+                        dialects: # for more info about dialects, see https://github.com/opencybersecurityalliance/stix-shifter/blob/develop/OVERVIEW.md
+                          - beats # need it if the index is created by Filebeat/Winlogbeat/*beat
                 config:
                     auth:
                         id: VuaCfGcBCdbkQm-e5aOx
@@ -145,7 +148,11 @@ class StixShifterInterface(AbstractDataSourceInterface):
             # STIX-shifter will alter the config objects, thus making them not reusable.
             # So only give STIX-shifter a copy of the configs.
             # Check `modernize` functions in the `stix_shifter_utils` for details.
-            (connector_name, connection_dict, configuration_dict,) = map(
+            (
+                connector_name,
+                connection_dict,
+                configuration_dict,
+            ) = map(
                 copy.deepcopy, get_datasource_from_profiles(profile, config["profiles"])
             )
 
@@ -203,19 +210,23 @@ class StixShifterInterface(AbstractDataSourceInterface):
 
                     result_retrieval_offset = 0
                     has_remaining_results = True
+                    lastsortvalue = None
                     while has_remaining_results:
                         result_batch = transmission.results(
-                            search_id, result_retrieval_offset, RETRIEVAL_BATCH_SIZE
+                            search_id,
+                            result_retrieval_offset,
+                            RETRIEVAL_BATCH_SIZE,
+                            lastsortvalue,
                         )
                         if result_batch["success"]:
                             new_entries = result_batch["data"]
                             if new_entries:
                                 connector_results += new_entries
                                 result_retrieval_offset += RETRIEVAL_BATCH_SIZE
-                                if len(new_entries) < RETRIEVAL_BATCH_SIZE:
-                                    has_remaining_results = False
                             else:
                                 has_remaining_results = False
+                            if "lastsort" in result_batch:
+                                lastsortvalue = result_batch["lastsort"]
                         else:
                             stix_shifter_error_msg = (
                                 result_batch["error"]
